#include "LPC17xx.h"
#include "FreeRTOS.h"
//#include "task.h"
//#include "semphr.h"
//#include "type.h"  //Included in uart.h
#include "uart.h"
#include "printf.h"
#include "wifly.h"
#include "config.h"
#include "configurations.h"
#include "comm.h"

#include <stdio.h>
#include <string.h>

static char *ssid = NETWORK_SSID;
static char *passwd = PASSWD;
static char *serverAdd = SERVER_ADDRESS;
static char *serverPort = SERVER_PORT;


/**********************************************************************************
 ** Function name:		debugInit
 **
 ** Description:		initializes debug UART
 **
 ** Parameters:			none
 ** Returned value:		none
 **********************************************************************************/

void debugInit()
{
	if(Print_Init(UART_DBG, BR_96))
	  Print_Text(UART_DBG, (char *)"Debug port initialized: OK\n");
}

/**********************************************************************************
 ** Function name:		wiflyInit
 **
 ** Description:		initializes wifly UART
 **
 ** Parameters:			none
 ** Returned value:		none
 **********************************************************************************/
void wiflyInit()
{
	if(Wifly_Init(UART_WF, BR_96))
		Print_Text(UART_DBG, (char *)"Wifly port initialized: OK\n");
	else
		Print_Text(UART_DBG, (char *)"Wifly port initialized: ERROR\n");
}

/**********************************************************************************
 ** Function name:		joinSSID
 **
 ** Description:		joins wifi network
 **
 ** Parameters:			none
 ** Returned value:		none
 **********************************************************************************/
void joinSSID()
{
	if(Wifly_JoinSSID(UART_WF, ssid, passwd, WPA1))
		Print_Text(UART_DBG, (char *)"Connected to wifi AP: OK\n");
	else
		Print_Text(UART_DBG, (char *)"Connected to wifi AP: ERROR\n");

}

/**********************************************************************************
 ** Function name:		openConn
 **
 ** Description:		opens remote connection
 **
 ** Parameters:			none
 ** Returned value:		none
 **********************************************************************************/
uint8_t openConn()
{
	char addPort[20];		//address + port
	uint32_t success;

	strcpy(addPort,serverAdd);
	strcat(addPort," ");
	strcat(addPort,serverPort);

	success = Wifly_Open(UART_WF, addPort);
	if (!success)
		Print_Text(UART_DBG, (char *)"Connected to server: ERROR\n");
	else
		Print_Text(UART_DBG, (char *)"Connected to server: OK\n");

	return(success);
}


/**********************************************************************************
 ** Function name:		connInit
 **
 ** Description:		joins wifi network + open remote connection
 **
 ** Parameters:			none
 ** Returned value:		none
 **********************************************************************************/
void connInit()
{
	uint8_t tries = 0;
	uint8_t success=FALSE;
	char message[10];		//message to server

	while(!success)
	{
		Wifly_Reset();
		Delay_Ms(DELAY, 500); // min 160us

		joinSSID();
		while (!success && tries < 4)
		{
			success =  openConn();
			tries++;
		}
		tries=0;
	}
	sprintf(message, "");
	Print_T_CRLF(UART_WF,message);
}

/**********************************************************************************
 ** Function name:		stablishConn
 **
 ** Description:		initializes connection & identifies board until success
 **
 ** Parameters:			none
 ** Returned value:		none
 **********************************************************************************/
void stablishConn(void)
{
	do
	{
		connInit();
	}
	while (!sendMessage(30,0)); //send identification message
}

/**********************************************************************************
 ** Task name:		sendMessage
 **
 ** Description:		sends a message of the indicated type
 **
 ** Parameters:			type of message
 ** 					second parameter depends on type of message
 ** 					      - beat rate
 ** 					      - battery status
 ** 					      - identification
 ** 					      - (ignored if not relevant for current message)
 ** Returned value:		status
 **********************************************************************************/
uint8_t sendMessage(uint8_t type, uint8_t param)
{
	char * ans;
	char message[20];			//message to server
	uint8_t status = TRUE;

	switch (type){
		/* beat rate message - 20
		 * param = beat rate
		 */
		case 20:
			sprintf(message, "%d:%3d", type, param);
			break;
		/* board identification message - 30
		 * param : ignored
		 */
		case 30:
			sprintf(message, "%d:%3d", type, BOARD_NUM);
			break;
		/* battery level message - 40
		 * param = battery status
		 */
		case 40:
			sprintf(message, "%d:%3d", type, param);
			break;
		/*
		 * Other message types
		 * 10 - Fall alarm
		 * param : ignored
		 */
		default:
			sprintf(message, "%d", type);
			break;
	}

	ans = Wifly_Send_And_Read_Response(UART_WF, message, 2000);

	if (strstr(ans, (char*)"OK")==NULL)
	{
		status=FALSE;
	}
	return status;
}
/**********************************************************************************
 ** Task name:		sendMessageCheckConn
 **
 ** Description:		sends a message and checks connection. Reestablishes connection if necessary
 **
 ** Parameters:			type of message
 ** 					second parameter depends on type of message
 ** 					      - beat rate
 ** 					      - battery status
 ** 					      - identification
 ** 					      - (ignored if not relevant for current message)
 ** Returned value:		none
 **********************************************************************************/
void sendMessageCheckConn(uint8_t type, uint8_t param)
{
	if (!sendMessage(type,param))
	{
		stablishConn();
	}
}


