/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

#include "LPC17xx.h"
/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"

/* Utils Libray includes and Declarations. */
#include "type.h"
#include "printf.h"
#include "wifly.h"
#include "adc.h"
#include "comm.h"

#include "config.h"
#include "stdio.h"
#include "string.h"
#include "configurations.h"

// Task declaration
static void vTaskHeartRate( void *pvParameters );
static void vTaskFall( void *pvParameters );
void systemInit(void);
void readBattery(void);

/* Declare a binary semaphore for fall task synchronization */
static xSemaphoreHandle xSemFall =NULL;

/**********************************************************************************
 ** Function name:		EINT3_IRQHandler
 **
 ** Description:		External Interrupt 3 Handler
 **
 ** Parameters:			None
 ** Returned value:		None
 ************************************************************************************/
void EINT3_IRQHandler()
{
	static portBASE_TYPE xHigherPriorityTaskWoken;
	xSemaphoreGiveFromISR(xSemFall,&xHigherPriorityTaskWoken);
	LPC_GPIOINT->IO0IntClr |= (1 << 4);		//Clear interrupt on P0[4]
}

/**********************************************************************************
 ** Function name:		main
 **
 ** Description:		main function
 **
 ** Parameters:			None
 ** Returned value:		int
 **********************************************************************************/
int main( void )
{
	/* Create binary semaphore for fall task sync */
	vSemaphoreCreateBinary(xSemFall);

	/* Only create the tasks if the semaphore was created successfully. */
	if (xSemFall != NULL)
	{
    	/* Task Creation */
    	xTaskCreate( vTaskHeartRate, (char *)"HeartRate", 240, NULL, 1, NULL);
    	xTaskCreate( vTaskFall, (char *)"FallDetection", 240, NULL, 3, NULL);
		/* Start the tasks running. */
		vTaskStartScheduler();
	}

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

/*****************************************************************************************
 ** Task name:		vTaskHeartRate
 **
 ** Description:		initializes peripherals
 ** 					monitors heart rate and sends to server
 ** 					monitors battery and sends to server
 **
 ** Parameters:			pointer to whatever structure desired (not used)
 ** Returned value:		none
 *****************************************************************************************/
static void vTaskHeartRate( void *pvParameters )
{
	uint16_t beatSignal=0;		//photopletysmograph signal level

	uint16_t timeTot=0;			//accumulated time (10 beats)
	uint16_t timeLastBeat=0;
	uint8_t countTreshold=0;	//signal>3000 counter
	uint8_t numBeats=0;
	uint8_t state=1; 			//counting state 	(0= ignore signals ; 1 = don't ignore them)
	uint32_t countBeatSign=0;	//counter of beat signals read. When 10000 we'll read battery level
	portTickType xLastWakeTime;

	systemInit();

	//time counter initialization
	xLastWakeTime = xTaskGetTickCount();

	for( ;; )
	{
		vTaskDelayUntil(&xLastWakeTime, 5/portTICK_RATE_MS);	//heart reading interval is 5 ms

		timeTot=timeTot+5;
		timeLastBeat=timeLastBeat+5;

		/* Read value in AD0.4*/
		while((LPC_ADC->ADDR4 >> 31) && 1){} //Wait until AD0.4 conversion
		beatSignal = (LPC_ADC->ADDR4>>4) & 0xFFF; //Read value

		countBeatSign++;
		//if we count a signal>3000 five consecutive times, in normal state, we have a beat
		if (beatSignal>BEAT_SIGN_TRESHOLD)
		{
			countTreshold++;

			if (countTreshold==5 && state)
			{
				numBeats++;
				/*start counting in the first beat*/
				if (numBeats==1)
				{
					timeTot=0;
				}
				//Calculate beat rate if we have 10 beats
				else if (numBeats==10)
				{
					sendMessageCheckConn(20,540000/timeTot);

					//before starting new round of 10 beat count, we reset times
					xLastWakeTime = xTaskGetTickCount();
					numBeats=0;
				}
				countTreshold=0;
				timeLastBeat=0;
				state=0;	//after counting a beat, we will ignore signals for a while
			}
		}
		else
		{
			countTreshold=0;
		}
		//250 ms after the last Beat, we stop ignoring signals
		if (timeLastBeat==NON_COUNT_INTERVAL)
			state=1;
		//if 2,5 seconds since last beat, send a message to the server with 0 beat/sec rate.
		if (timeLastBeat==ZERO_BEAT_INTERVAL)
		{
			sendMessageCheckConn(20,0);

			//before starting new round of 10 beat count, we reset times
			numBeats=0;
			timeLastBeat=0;
			xLastWakeTime = xTaskGetTickCount();
		}
		//every 10000 signals, read battery
		if (countBeatSign==BATTERY_INTERVAL)
		{
			countBeatSign=0;

			readBattery();

			//before starting new round of 10 beat count, we reset times
			xLastWakeTime = xTaskGetTickCount();
			timeLastBeat=0;
			numBeats=0;
		}
	}
}


/**********************************************************************************
 ** Task name:			System Init
 **
 ** Description:		Initializes required hardware
 **
 ** Parameters:			none
 ** Returned value:		none
 **********************************************************************************/
void systemInit(void){

	debugInit();
	wiflyInit();

	//joinSSID();
	stablishConn();

	adcInit();		//Initializes ADC for 0g and battery level detection
	adcChannel(4); 	//Set channel 4 for AD0.4
	adcChannel(1); //Set channel 1 for AD0.1

	//Enables P0.4 as input GPIO pin
	LPC_PINCON->PINSEL0 &=~((1<<9)|(1<<8));
	LPC_GPIO0->FIODIR &= ~(1<<4);

	LPC_GPIOINT->IO0IntEnR |= (1<<4);	//Initialize interruption for P0.4 in rising flank
	NVIC_EnableIRQ(EINT3_IRQn);			//Enable interrupt number 3
}

/**********************************************************************************
 ** Task name:			readBattery
 **
 ** Description:		read and sends battery level
 **
 ** Parameters:			none
 ** Returned value:		none
 **********************************************************************************/
void readBattery(void){

	uint16_t batterySignal=0;
	uint8_t batteryState=0;

	//read battery level
	while((LPC_ADC->ADDR1 >> 31) && 1){}
	batterySignal = (LPC_ADC->ADDR1>>4) & 0xFFF;

	if (batterySignal<(PERCENTAGE_BATT_TRESHOLD*MAX_SIGNAL))
	{
		batteryState=0;
	}
	else if (batterySignal>=(PERCENTAGE_BATT_TRESHOLD*MAX_SIGNAL))
	{
		batteryState=1;
	}
	sendMessageCheckConn(40,batteryState);
}
/**********************************************************************************
 ** Task name:			vTaskFall
 **
 ** Description:		handle fall condition.
 **
 ** Parameters:			pointer to whatever structure desired
 ** Returned value:		none
 **********************************************************************************/
static void vTaskFall( void *pvParameters )
{
	xSemaphoreTake(xSemFall,portMAX_DELAY);
	for (;;)
	{
		xSemaphoreTake(xSemFall,portMAX_DELAY);
		while (!sendMessage(10,0)) //send fall alarm message
		{
			stablishConn();
		}
	}
}

/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	Print_T_CRLF(UART3, (char *)"Error de Malloc Failed");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	Print_T_CRLF(UART3, (char *)"Error de Stack Overflow");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}
