/*
 * printf.c
 *
 *  Created on: 25/08/2013
 *      Author: Xavier Vilajosana
 */
#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "uart.h"
#include "printf.h"
#include "config.h"
#include <stdio.h>
#include <string.h>


extern volatile uint32_t UART0Count, UART1Count, UART2Count, UART3Count;
extern volatile uint8_t UART0Buffer[BUFSIZE], UART1Buffer[BUFSIZE], UART2Buffer[BUFSIZE], UART3Buffer[BUFSIZE];

/* Declare a variable of type xSemaphoreHandle */
volatile static xSemaphoreHandle pMutex = NULL;

/*****************************************************************************
** Function name:		DelayMs
**
** Descriptions:		Procedure implementing an active wait based on a simple loop
**
** parameters:			tipo(Brute or Task), ms miliseconds to delay
** Returned value:		None
**
*****************************************************************************/
void Delay_Ms(uint8_t base, uint32_t ms)
{
	if (base == 0) // Active Delay
	{
		uint32_t i;
		for(i=0; i<(((uint32_t)7140)*ms);i++); // loop 7140 approximate 1 millisecond
	}
	else if (base == 1)  // Passive Delay
	{
		const portTickType xDelayMs = (portTickType) ms / portTICK_RATE_MS;
		vTaskDelay(xDelayMs);
	}
}

/*****************************************************************************
** Function name:		Lock_Print
**
** Descriptions:		Use Semaphore or vTaskSuspend for block process
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void Lock_Print()
{
	if (LOCK == 2)
	{
		if(pMutex == NULL)
		{
			pMutex = xSemaphoreCreateMutex();
		}
		xSemaphoreTake(pMutex, portMAX_DELAY);
	}
	else if (LOCK == 1)
	{
		vTaskSuspendAll();
	}
}

/*****************************************************************************
** Function name:		UnLock_Print
**
** Descriptions:		Use Semaphore or vTaskResume for unblock process
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void UnLock_Print()
{
	if (LOCK == 2)
	{
		xSemaphoreGive(pMutex);
	}
	else if (LOCK == 1)
	{
		xTaskResumeAll();
	}
}

/*****************************************************************************
** Function name:		Print_Flush
**
** Descriptions:		Erase the buffer used in UART port.
**
** parameters:			UART port
** Returned value:		None
**
*****************************************************************************/
void Print_Flush(uint8_t PortNum)
{
	UART_Flush(PortNum);
}


/*****************************************************************************
** Function name:		Print_Init
**
** Descriptions:		Initialize the UART Print connection
**
** parameters:			UART port, Baudrate to connect
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint32_t Print_Init(uint8_t PortNum, uint32_t Baudrate)
{
	uint32_t ret = FALSE;
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Print();
	{
		ret = UART_Init(PortNum, Baudrate);
		UART_Flush (PortNum);
	}
	UnLock_Print();
	return ret;
}

/*****************************************************************************
** Function name:		Print_End
**
** Descriptions:		Close the UART Print connection
**
** parameters:			UART port
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint32_t Print_End(uint8_t PortNum)
{
	uint32_t ret = FALSE;
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Print();
	{
		ret = UART_Close(PortNum);
		UART_Flush (PortNum);
	}
	UnLock_Print();
	return ret;
}

/*****************************************************************************
** Function name:		Print_Text
**
** Descriptions:		Print a text
**
** parameters:			UART port, Pointer to the text to be printed
** Returned value:		None
**
*****************************************************************************/
void Print_Text (uint8_t PortNum , char *Text)
{
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Print();
	{
		UART_Send(PortNum, Text);
	}
	UnLock_Print();
}

/*****************************************************************************
** Function name:		Print_T_CRLF
**
** Descriptions:		Print a text with type of end line
**
** parameters:			UART port, Pointer to the text to be printed
** Returned value:		None
**
*****************************************************************************/
void Print_T_CRLF (uint8_t PortNum , char *Text)
{
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Print();
	{
		UART_Send(PortNum, Text);
		UART_Send(PortNum, (char *)"\r\n");
	}
	UnLock_Print();
}

/*****************************************************************************
** Function name:		Print_TextNum
**
** Descriptions:		Print a text and a number and a word passed as a parameter
**
** parameters:			Pointer to the text, number value
** Returned value:		None
**
*****************************************************************************/

void Print_TT_CRLF( uint8_t PortNum , char *Text , char *Word)
{
	char buf[GetSIZE];
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Print();
	{
		sprintf(buf, "%s %s\r\n", Text, Word);
		UART_Send(PortNum, (char*)buf);
	}
	UnLock_Print();
}

/*****************************************************************************
** Function name:		Print_TextNum
**
** Descriptions:		Print a text and a number passed as a parameter
**
** parameters:			Pointer to the text, number value
** Returned value:		None
**
*****************************************************************************/

void Print_TextNum ( uint8_t PortNum , char *Text , int32_t Num)
{
	char buf[GetSIZE];
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Print();
	{
		sprintf(buf, "%s %i", Text, Num);
		UART_Send(PortNum, (char *)buf);
	}
	UnLock_Print();
}

/*****************************************************************************
** Function name:		Print_TN_CRLF
**
** Descriptions:		Print a text and a number with type of end line
**
** parameters:			Pointer to the text, number value, type of end line
** Returned value:		None
**
*****************************************************************************/

void Print_TN_CRLF ( uint8_t PortNum , char *Text , int32_t Num)
{
	char buf[GetSIZE];
	/* Verify a Mutex type semaphore to continue printf */
	Lock_Print();
	{
		sprintf(buf, "%s %i\r\n", Text, Num);
		UART_Send(PortNum, (char *)buf);
	}
	UnLock_Print();
}

/*****************************************************************************
** Function name:		UART_Scanf
**
** Descriptions:		Read the UART port waiting time parameter
**
** parameters:			UART Port, Pointer to Buffer and Time to wait in milliseconds
** Returned value:		Number of characters read
**
*****************************************************************************/
uint32_t UART_Scanf(uint8_t PortNum, char *Text, uint32_t Time)
{
	uint32_t oldCount = 0;
	uint32_t newCount = 0;
	{
		if ( PortNum == 0 )
		{
			LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
			Delay_Ms(DELAY,Time); // wait sometime for receive some commands
			do{
				oldCount = newCount;
				Delay_Ms(DELAY,5);
				newCount = UART0Count;
			}while(oldCount != newCount); // wait a little time while we have new chars
			LPC_UART0->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
			strncpy(Text, (char *)UART0Buffer, newCount ); // Obtain UART data
		}
		else if ( PortNum == 1 )
		{
			LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
			Delay_Ms(DELAY,Time); // wait sometime for receive some commands
			do{
				oldCount = newCount;
				Delay_Ms(DELAY,5);
				newCount = UART1Count;
			}while(oldCount != newCount); // wait a little time while we have new chars
			LPC_UART1->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
			strncpy(Text, (char *)UART1Buffer, newCount ); // Obtain UART data
		}
		else if ( PortNum == 2 )
		{
			LPC_UART2->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
			Delay_Ms(DELAY,Time); // wait sometime for receive some commands
			do{
				oldCount = newCount;
				Delay_Ms(DELAY,5);
				newCount = UART2Count;
			}while(oldCount != newCount); // wait a little time while we have new chars
			LPC_UART2->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
			strncpy(Text, (char *)UART2Buffer, newCount ); // Obtain UART data
		}
		else if ( PortNum == 3 )
		{
			LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
			Delay_Ms(DELAY,Time); // wait sometime for receive some commands
			do{
				oldCount = newCount;
				Delay_Ms(DELAY,5);
				newCount = UART3Count;
			}while(oldCount != newCount); // wait a little time while we have new chars
			LPC_UART3->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0
			strncpy(Text, (char *)UART3Buffer, newCount ); // Obtain UART data
		}
		UART_Flush(PortNum); // Erase and initialize UART buffer
	}
	Text[newCount] = '\0'; // add end string. Now we have a string
	//#ifdef MODE_DEBUG
		Print_Text(0, Text);
	//#endif
	return (newCount);
}

/*****************************************************************************
** Function name:		UART_Read
**
** Descriptions:		Read the UART port waiting time parameter
**
** parameters:			UART Port, Pointer to Buffer and Time to wait in milliseconds
** Returned value:		Number of characters read
**
*****************************************************************************/
uint32_t UART_Read(uint8_t PortNum, char *Text, char *Value, uint32_t Time)
{
	Text[0] = '\0';
	while (Time > 0)
	{
		if ((PortNum == 0) && (UART0Count != 0))
		{
			LPC_UART0->IER = IER_THRE | IER_RLS;				/* Disable RBR */
			if (strlen((char *)UART0Buffer) + strlen(Text) < GetSIZE)
			{
				strncat(Text, (char*)UART0Buffer, UART0Count);
			}
			else Text = (char *)Overflow;
			UART0Count = 0;
			LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			if(strstr(Text, Value) != NULL) Time = 10;
		}
		else if ((PortNum == 1) && (UART1Count != 0))
		{
			LPC_UART1->IER = IER_THRE | IER_RLS;				/* Disable RBR */
			if (strlen((char *)UART1Buffer) + strlen(Text) < GetSIZE)
			{
				strncat(Text, (char*)UART1Buffer, UART1Count);
			}
			else Text = (char *)Overflow;
			UART1Count = 0;
			LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			if(strstr(Text, Value) != NULL) Time = 10;
		}
		else if ((PortNum == 2) && (UART2Count != 0))
		{
			LPC_UART2->IER = IER_THRE | IER_RLS;				/* Disable RBR */
			if (strlen((char *)UART2Buffer) + strlen(Text) < GetSIZE)
			{
				strncat(Text, (char*)UART2Buffer, UART2Count);
			}
			else Text = (char *)Overflow;
			UART2Count = 0;
			LPC_UART2->IER = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			if(strstr(Text, Value) != NULL) Time = 10;
		}
		else if ((PortNum == 3) && (UART3Count != 0))
		{
			LPC_UART3->IER = IER_THRE | IER_RLS;				/* Disable RBR */
			if (strlen((char *)UART3Buffer) + strlen(Text) < GetSIZE)
			{
				strncat(Text, (char*)UART3Buffer, UART3Count);
			}
			else Text = (char *)Overflow;
			UART3Count = 0;
			LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;		/* Re-enable RBR */
			if(strstr(Text, Value) != NULL) Time = 10;
		}
		Delay_Ms(DELAY, 1);
		Time--;
	}
	UART_Flush(PortNum);
#ifdef MODE_DEBUG
	Print_Text(UART_DBG, Text);
#endif
	return strlen(Text);
}

/*****************************************************************************
** Function name:		Check_Read
**
** Descriptions:		Read the UART port waiting time parameter and check with value
**
** parameters:			UART Port, Pointer to Buffer and Time to wait in milliseconds
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/

uint32_t Check_Read(uint8_t PortNum, char *Text, char *Value, uint32_t Time)
{
	uint32_t ret = FALSE;
	if (UART_Read(PortNum, Text, Value, Time) > 0)
	{
		if (strstr(Text, Value) != NULL) ret = TRUE;
	}
	return ret;
}
