/*
 * wifly.c
 *
 *  Created on: 25/08/2013
 *      Author: Xavier Vilajosana
 */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "type.h"
#include "printf.h"
#include "wifly.h"
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "LPC17xx.h"

static char Buffer[GetSIZE];
static uint32_t Estat = NORMAL, Error_Send = 0, Error_Read = 0;

/* Declare a variable of type xSemaphoreHandle */
volatile static xSemaphoreHandle wMutex = NULL;

/*****************************************************************************
 ** Function name:		Lock_Wifly
 **
 ** Descriptions:		Use Semaphore or vTaskSuspend for block process
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void Lock_Wifly() {
	if (wMutex == NULL ) {
		wMutex = xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive(wMutex, portMAX_DELAY);
}

/*****************************************************************************
 ** Function name:		UnLock_Wifly
 **
 ** Descriptions:		Use Semaphore or vTaskResume for unblock process
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void UnLock_Wifly() {
	xSemaphoreGiveRecursive(wMutex);
}

/*****************************************************************************
 ** Function name:		Wifly_Reset
 **
 ** Descriptions:		Hardware reset the WiFly device. PIN 6-24 (P0.22) of LPC1769
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void Wifly_Reset() {
	// Initialize pin
	LPC_PINCON ->PINSEL1 &= (~(3 << 12));
	LPC_GPIO0 ->FIODIR |= (1 << WIFLY_PIN_RST);
	// it made reset at Wifly
	LPC_GPIO0 ->FIOCLR = 1 << WIFLY_PIN_RST;
	Delay_Ms(DELAY, 500); // min 160us
	LPC_GPIO0 ->FIOSET = 1 << WIFLY_PIN_RST;
}

/*****************************************************************************
 ** Function name:		Wifly_Init
 **
 ** Descriptions:		Defines the used UART, its baudrate and reset device
 ** 						This operation produce a few delay.
 **
 ** parameters:			UART port ,  desired baudrate
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Init(uint8_t PortNum, uint32_t Baudrate) {
	uint32_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Wifly();
	{
		Print_Flush(PortNum);
		ret = Print_Init(PortNum, Baudrate);
		//Wifly_Reset();
		Delay_Ms(DELAY, 1000);  // min 160us
		Print_Flush(PortNum);
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_End
 **
 ** Descriptions:		Close the UART port used for WiFly device.
 **
 ** parameters:			UART port
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_End(uint8_t PortNum) {
	uint32_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Wifly();
	{
		Print_Flush(PortNum);
		ret = Print_End(PortNum);
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set(uint8_t PortNum, char *Comand, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, Comand);
	if (UART_Read(PortNum, (char *) Buffer, (char *) "\n", Time) > 0)
		ret = TRUE;
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_OK(uint8_t PortNum, char *Comand, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, Comand);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TT_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer to join at command
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TT_OK(uint8_t PortNum, char *Comand, char *Value,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_TT_CRLF(PortNum, Comand, Value);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TT_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, number to join at command
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TN_OK(uint8_t PortNum, char *Comand, int32_t Num,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_TN_CRLF(PortNum, Comand, Num);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_Check
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer to check
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_Check(uint8_t PortNum, char *Comand, char *Check,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, Comand);
	ret = Check_Read(PortNum, (char *) Buffer, Check, Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Send_And_Read_Response
 **
 ** Descriptions:		Envia dades a WiFly i en retorna la resposta
 **
 ** parameters:			port UART, text a enviar, resposta i temps per esperar la resposta
 **
 ** Returned value:		Valor llegit
 **
 *****************************************************************************/
char *Wifly_Send_And_Read_Response(uint8_t PortNum,char *Send, uint32_t Time){
	Lock_Wifly();
	Print_Flush(PortNum);
	Print_T_CRLF(PortNum, Send);
	//El servidor sempre respon amb OK. Per tant aquest es el missatge esperat
	Check_Read(PortNum, (char *) Buffer, (char *) "OK", Time);
	UnLock_Wifly();
	return (char *)Buffer;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TT_Check
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer Value to join at command
 ** 						char pointer to Check, and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TT_Check(uint8_t PortNum, char *Comand, char *Value,
		char *Check, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_TT_CRLF(PortNum, Comand, Value);
	ret = Check_Read(PortNum, (char *) Buffer, Check, Time);
	Delay_Ms(DELAY, 1000);
	UnLock_Wifly();
	return ret;
}

char* Wifly_Set_TT_Check2(uint8_t PortNum, char *Comand, char *Value,
		char *Check, uint32_t Time) {
	Lock_Wifly();
	Print_Flush(PortNum);
	Print_TT_CRLF(PortNum, Comand, Value);
	Check_Read(PortNum, (char *) Buffer, (char *)Check, Time);
	Delay_Ms(DELAY, 1000);
	UnLock_Wifly();
	return Buffer;
}

/*****************************************************************************
 ** Function name:		Wifly_CMD
 **
 ** Descriptions:		Send '$$$' to enter at Command Mode of WiFly
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_CMD(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == COMMAND)
		return TRUE;
	Print_Flush(PortNum);
	Print_Text(PortNum, (char *) "$$$");
	//sRet = Check_Read2(PortNum, (char *) Buffer, (char *) "CMD", 300);
	Check_Read(PortNum, (char *) Buffer, (char *) "CMD", 300);
	if (strstr(Buffer,(char *)"CMD")!=NULL)
	{
		Estat = COMMAND;
		ret = TRUE;
	}
	else
		ret = FALSE;
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Exit
 **
 ** Descriptions:		Send 'exit' to return at Serial Mode of WiFly
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Exit(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Open
 **
 ** Descriptions:		Send 'open' with Address (empty or text)
 **
 ** parameters:			UART port to send and Address to connect
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Open(uint8_t PortNum, char *Address) {
	uint32_t ret = FALSE;
	char * sRet;

	 Lock_Wifly();
	 Print_Flush(PortNum);

	 //Send OPEN command
	 sRet = Wifly_Set_TT_Check2(PortNum, (char *) "open", Address, (char *) "*OPEN*",2000);

	 if (strstr(sRet,(char *)"*OPEN*")!=NULL)
	 {
	 	 ret = TRUE;
	 	 Estat = NORMAL;
	 }
	 else
	 	ret = FALSE;
	 UnLock_Wifly();
	 return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Close
 **
 ** Descriptions:		Send 'close' to close open TCP connection
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Close(uint8_t PortNum) {
	uint32_t ret = FALSE;
	 //lock wifly
	 Lock_Wifly();
	 //flush de la uart
	 Print_Flush(PortNum);
	 //Send close
	 Print_T_CRLF(PortNum,(char *) "close");
	 Estat = NORMAL;
	 //if (ret == TRUE) Estat = NORMAL;
	 UnLock_Wifly();
	 return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Save
 **
 ** Descriptions:		Send 'save' to store actual configuration
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Save(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Reboot
 **
 ** Descriptions:		Send 'reboot' now initialize the WiFly
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Reboot(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Connect
 **
 ** Descriptions:		Connect to a Wi-Fy network
 **
 ** parameters:			Pointer to Buffer and Time to wait in milliseconds
 ** Returned value:		Number of characters read
 **
 *****************************************************************************/
uint32_t Wifly_Connect(uint8_t PortNum, uint32_t Mode, char *ssid, char* Phrase) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	Wifly_Set_TN_OK(PortNum, (char *) "set w a", Mode, 500);
	Wifly_Set_TT_OK(PortNum, (char *) "set w s", ssid, 500);
	Wifly_Set_TT_OK(PortNum, (char *) "set w p", Phrase, 750);
	ret = Wifly_Set_TT_Check(PortNum, (char *) "join", ssid,
			(char *) "Associated!", 5000);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_AutoConnect
 **
 ** Descriptions:		Connect to a Wi-Fy network mode Auto at Start
 **
 ** parameters:			Pointer to Buffer and Time to wait in milliseconds
 ** Returned value:		Number of characters read
 **
 *****************************************************************************/
uint32_t Wifly_AutoConnect(uint8_t PortNum, uint32_t Mode, char* Phrase) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	Wifly_Set_TN_OK(PortNum, (char *) "set w a", Mode, 500);
	Wifly_Set_TT_OK(PortNum, (char *) "set w p", Phrase, 750);
	Wifly_Set_OK(PortNum, (char *) "set w j 1", 500);
	ret = Wifly_Set_Check(PortNum, (char *) "join", (char *) "Associated!",
			5000);
	if (ret == TRUE)
		ret = Wifly_Save(PortNum);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Disconnect
 **
 ** Descriptions:		Send 'leave' to disconnect the WLAN
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/

uint32_t Wifly_Disconnect(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char*) "leave", (char *) "DeAuth", 500);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_JoinSSID
 **
 ** Descriptions:		Joins an SSID
 **
 ** parameters:			UART port to send, SSID to join
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_JoinSSID(uint8_t PortNum, char * ssid, char * passwd, uint8_t Mode ) {
	uint32_t ret = FALSE;
	char* sRet;
	Lock_Wifly();
	Wifly_CMD(PortNum);
	switch (Mode)
	{
		case WPA1:
			Print_TT_CRLF(PortNum, (char *)"set wlan phrase", passwd);
			break;
		case WEP128:
			Print_TT_CRLF(PortNum, (char *)"set wlan key", passwd);
			break;
	}
	UnLock_Wifly();
	Print_Flush(PortNum);
	sRet = Wifly_Set_TT_Check2(PortNum, (char *) "join", ssid, (char *) "Associated!",100000);
	UnLock_Wifly();
	if (strstr(sRet,(char *)"Associated!")!=NULL)
		ret = TRUE;
	else
		ret = FALSE;
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Ping
 **
 ** Descriptions:		Send 'ping' with Address
 **
 ** parameters:			UART port to send and Address to check ping
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Ping(uint8_t PortNum, char *Address) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	ret = Wifly_Set_TT_Check(PortNum, (char *) "ping", Address, (char *) "PING",
			2000);
#ifdef MODE_DEBUG
	if (ret == TRUE) {
		char *string;
		string = strstr(Buffer, (char *) "PING");
		string = strtok(string, (char *) "\r\n");
		Print_TT_CRLF(UART_DBG, (char *) "Ping result :", string);
	}
#endif
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_IP
 **
 ** Descriptions:		Send 'get ip' to know the IP used in WLAN
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_IP(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Channel
 **
 ** Descriptions:		Send 'show net' to know the Channel used in WLAN
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Channel(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_RSSI
 **
 ** Descriptions:		Send 'show rssi' to know the signal RSSI in dBm
 **
 ** parameters:			UART port to send
 ** Returned value:		RSSI Value or 0 if not found
 **
 *****************************************************************************/
int32_t Wifly_RSSI(uint8_t PortNum) {
	int32_t ret = 0;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_UpTime
 **
 ** Descriptions:		Send 'show time' to know the signal RSSI in dBm
 **
 ** parameters:			UART port to send
 ** Returned value:		UpTime Value or 0 if not found
 **
 *****************************************************************************/
int32_t Wifly_UpTime(uint8_t PortNum) {
	uint32_t ret = 0;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_GetPower
 **
 ** Descriptions:		Send 'get wlan' to know the transmission Power
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_GetPower(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_SetPower
 **
 ** Descriptions:		Send 'set wlan tx XX' to change the transmission Power
 ** 						where XX is a range 0-12. Default 0 = 12 dBm (Max Value)
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_SetPower(uint8_t PortNum, uint32_t Value) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_ChangeBR
 **
 ** Descriptions:		Change instantaneous Baudrate with 'set uart instant <value>'
 **
 ** parameters:			UART port to send , new desired Baudrate
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/

uint32_t Wifly_ChangeBR(uint8_t PortNum, int32_t Value) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_GET_Telnet
 **
 ** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.0 or HTTP/1.1
 **
 ** parameters:			UART port to send, HOST to connect, sub-address in Host, Mode HTTP
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_GET_Telnet(uint8_t PortNum, char *Host, char *Param) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_GET_Base
 **
 ** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.0 or HTTP/1.1
 **
 ** parameters:			UART port to send, HOST to connect, sub-address in Host, Mode HTTP
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_GET_Base(uint8_t PortNum, char *Host, char *Param) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

uint32_t Wifly_TCP(uint8_t PortNum, uint32_t value) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	Print_Text(PortNum,(char *)"Pole");
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Web_Send
 **
 ** Descriptions:		Connect to specified Host and parameters (CGI address), and add Value to send
 ** 						with Mode HTTP/1.1
 **
 ** parameters:			UART port to send, HOST to connect, sub-address in Host, Value to add
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Web_Send(uint8_t PortNum, char *Host, char *Param,
		uint32_t Value) {
	 Print_T_CRLF(UART_DBG,"Hola");
	 Print_T_CRLF(UART_DBG,"Fins aqui arribo");
	 uint32_t ret = FALSE;
	 char buf[SendSIZE];
	 //lock wifly
	 Lock_Wifly();
	 //enter cmd
	 //Wifly_CMD(PortNum);
	 //format url
	 //sprintf(buf, "%s %s", Host, (char *)NetPORT);
	 //open the url. Call wifly_open
	// Wifly_Open(PortNum,(char *)"arpalab.appspot.com 80");
	 //flush the uart

	 Print_Flush(PortNum);
	 //format the GET query
	 //sprintf(buf, "%s %s%u %s\r\nhost: %s\r\n\r\n", (char *)"GET", Param, Value, (char *)WEB, Host);

	 sprintf(buf, "GET /rest/stats?nid=123456&ip=213.22.111.2&app=test&data=1.345 HTTP/1.1\r\nhost: arpalab.appspot.com\r\n\r\n");
	 //print the buffer to the WiFLY
	 Print_T_CRLF(PortNum, buf);
	 //wait for wifly answer, this will return "200 OK" + result of the GET
	 ret = Check_Read(PortNum, (char *)Buffer,(char *)"200 OK", 5000);

	#ifdef MODE_DEBUG
	 if (ret == TRUE)
	 {
	  char *string;
	  string = strstr(Buffer, (char *)"HTTP");
	  string = strtok(string, (char *)"\r\n");
	  Print_TT_CRLF(UART_DBG, (char *)"Resultado de Web Send es :", string);
	 }
	#endif
	 if (ret == TRUE) Error_Send = 0;
	 else  Error_Send++;
	// Wifly_Close(PortNum);
	 UnLock_Wifly();
	 return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Web_Read
 **
 ** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.1
 ** 						and collect received Value.
 **
 ** parameters:			UART port to send, HOST to connect, sub-address in Host
 ** Returned value:		Value received or 0 if not correct
 **
 *****************************************************************************/
int32_t Wifly_Web_Read(uint8_t PortNum, char *Host, char *Param) {
	uint32_t ret = 0;
	 char buf[GetSIZE];
	 char *parse;
	 //lock wifly
	 Lock_Wifly();
	 //enter CMD
	 Wifly_CMD(PortNum);
	 //format host and port with sprintf
	 sprintf(buf, "%s %s", Host, (char *)NetPORT);
	 //open wifly connection -- call wifly_open
	 Wifly_Open(PortNum,(char *)"arpalab.appspot.com 80");
	 //flush printf to remove any pending char on the buffer
	 Print_Flush(PortNum);
	 //format url
	 sprintf(buf, "%s %s %s\r\nhost: %s\r\n\r\n", (char *)"GET", Param, (char *)WEB, Host);
	 //print the buffer containing the url to the wifly
	 Print_T_CRLF(PortNum, buf);
	 //this should return 200 OK in less than 5000ms
	 if (Check_Read(PortNum, (char *)Buffer,(char *)"200 OK", 5000) == TRUE)
	 {
	  strcpy(buf, Buffer);
	 #ifdef MODE_DEBUG
	  char *string;
	  string = strstr(Buffer, (char *)"HTTP"); //string points to the first occurrence of HTTP in the Buffer
	  string = strtok(string, (char *)"\r\n"); //tokenize the string (this removes \r\n)
	  Print_TT_CRLF(UART_DBG, (char *)"Resultado de Web Read es :", string);
	 #endif
	  parse = strstr(buf, (char *)"chunked"); //points to the first ocurrence of "chunked"
	  if  (parse == NULL)
	  {
	   parse = strstr(buf, (char *)"private"); //string tokenize, split by "private"
	   parse = strtok(parse, (char *)"*CLOS*");//string tokenize, split by "*CLOS*"
	   parse = strtok(parse, (char *)"\r\n");//tokenize the string (this removes \r\n)
	   parse = strtok(NULL, (char *)"\r\n");//tokenize the string (this removes \r\n)
	  }
	  else
	  {
	   parse = strtok(parse, (char *)"\r\n");//tokenize the string (this removes \r\n)
	   parse = strtok(NULL, (char *)"\r\n");//tokenize the string (this removes \r\n)
	   parse = strtok(NULL, (char *)"\r\n");//tokenize the string (this removes \r\n)
	  }
	  ret = atoi(parse);
	  Error_Read = 0;
	 }
	 else{
	     Error_Read++;
	 }
	 Wifly_Close(PortNum);
	 UnLock_Wifly();
	 return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Send_Error
 **
 ** Descriptions:		Check the number of errors in send function and Initialize value.
 **
 ** parameters:			Limit to check and True o false for initialize
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Send_Error(uint32_t Limit, uint32_t Init) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	{
		if (Error_Send > Limit)
			ret = TRUE;
		if (Init == TRUE)
			Error_Send = 0;
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Read_Error
 **
 ** Descriptions:		Check the number of errors in read function and Initialize value.
 **
 ** parameters:			Limit to check and True o False for initialize
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Read_Error(uint32_t Limit, uint32_t Init) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	{
		if (Error_Read > Limit)
			ret = TRUE;
		if (Init == TRUE)
			Error_Read = 0;
	}
	UnLock_Wifly();
	return ret;
}
